// Include the libraries we need
#include <OneWire.h>
#include <DallasTemperature.h>
#include <Wire.h>
#include <LiquidCrystal_I2C.h>


#define ONE_WIRE_BUS 8
#define Pomp 3
#define buzzer 5
#define LED_GREEN 6
#define LED_RED 7
OneWire oneWire(ONE_WIRE_BUS);
LiquidCrystal_I2C lcd(0x27, 16, 2);  // set the LCD address to 0x27 for a 16 chars and 2 line display
DallasTemperature sensors(&oneWire);


float tempC = 0;
float my_timer = 0;
bool first_measure = true;
bool fire_zone = false;

ISR(TIMER1_COMPA_vect) {
  my_timer++;
}

void configure_timer1() {
  TCCR1A = 0;
  TCCR1B = 0;
  TCNT1 = 0;
  OCR1A = 31249;
  TCCR1B |= (1 << WGM12);
  TCCR1B |= (1 << CS12);
}

void init_timer1() {
  TIMSK1 |= (1 << OCIE1A);
}

void setup(void) {
  // start serial port
  Serial.begin(9600);
  Serial.println("Dallas Temperature IC Control Library Demo");

  // Start up the library
  sensors.begin();
  pinMode(Pomp, OUTPUT);
  pinMode(buzzer, OUTPUT);
  pinMode(LED_GREEN, OUTPUT);
  pinMode(LED_RED, OUTPUT);

  Serial.begin(9600);

  digitalWrite(LED_GREEN, HIGH);
  digitalWrite(LED_RED, LOW);
  digitalWrite(Pomp, HIGH);
  digitalWrite(buzzer, LOW);
  lcd.init();  // initialize the lcd
  // Print a message to the LCD.
  lcd.backlight();
  lcd.setCursor(2, 0);
  lcd.print("Sistemul e ON");

  cli();
  configure_timer1();
  init_timer1();

  sei();
}

/*
 * Main function, get and show the temperature
 */
void loop(void) {

  if (Serial.available() > 0) {
    String input = Serial.readString();
    input.trim();
    if (input.equals("start")) {
      lcd.clear();

      lcd.setCursor(0, 0);
      lcd.print("Stinge focul!");
      digitalWrite(buzzer, HIGH);
      digitalWrite(Pomp, LOW);
      digitalWrite(LED_RED, HIGH);
      digitalWrite(LED_GREEN, LOW);
    } else if (input.equals("stop")) {
      lcd.clear();
      lcd.setCursor(0, 0);
      lcd.print("Incendiul a");
      lcd.setCursor(0, 1);
      lcd.print("fost stins!");

      digitalWrite(buzzer, LOW);
      digitalWrite(Pomp, HIGH);
      digitalWrite(LED_RED, LOW);
      digitalWrite(LED_GREEN, HIGH);

      first_measure = true;
    }
  }

  if (my_timer >= 20 || first_measure == true) {
    sensors.requestTemperatures();
    tempC = sensors.getTempCByIndex(0);
    delay(1000);

    if (tempC > 26) {
      lcd.clear();
      lcd.setCursor(3, 0);
      lcd.print("Alerta");
      lcd.setCursor(4, 1);
      lcd.print("Foc!");
      digitalWrite(buzzer, HIGH);
      digitalWrite(Pomp, LOW);

      digitalWrite(LED_RED, HIGH);
      digitalWrite(LED_GREEN, LOW);
      fire_zone = true;
    } else {
      if (fire_zone == true) {
        fire_zone = false;
        digitalWrite(buzzer, LOW);
        digitalWrite(Pomp, HIGH);
        digitalWrite(LED_RED, LOW);
        digitalWrite(LED_GREEN, HIGH);
      }
      lcd.clear();
      lcd.setCursor(0, 0);
      lcd.print("Temperatura este");
      lcd.setCursor(3, 1);
      lcd.print("in regula!");
      delay(1000);
    }
    // Check if reading was successful
    if (tempC != DEVICE_DISCONNECTED_C) {
      Serial.print("Temperature for the device 1 (index 0) is: ");
      Serial.println(tempC);
    } else {
      Serial.println("Error: Could not read temperature data");
    }

    my_timer = 0;
    first_measure = false;
  }
}